# Bu araç @keyiflerolsun tarafından | @KekikAkademi için yazılmıştır.

from tabulate import tabulate
from typing   import List, Dict

def list2html(title:str, tablo_veri:List[Dict], header:str='', aciklama:str='', footer:str='', pdf_sayfa:str="A4", sayfalama:bool=True) -> str:
    tablo_html = tabulate(tablo_veri, headers='keys', tablefmt='html')

    css = """<style>
        th, td {
            text-align: center !important;
        }
    </style>"""

    javascript = """<script>
            $("table").addClass("table table-bordered table-sm table-hover table-striped");
            $("thead").addClass("thead-dark");
            $(document).ready(function() {
                $("table").DataTable({
                    dom: 'Bfrtip',
                    paging: false,
                    language: {
                        "emptyTable": "Tabloda veri yok",
                        "lengthMenu": "Sayfa başına _MENU_ kaydı görüntüle",
                        "zeroRecords": "Hiçbir eşleşen kayıt bulunamadı",
                        // "info": "_TOTAL_ girişten _START_ - _END_ arası gösteriliyor",
                        "info": "_TOTAL_ Kayıt Mevcut",
                        "infoEmpty": "Kayıt yok",
                        "infoFiltered": "(_MAX_ toplam kayıttan filtrelenmiştir)",
                        "search": "Ara:",
                        "paginate": {
                            "first":      "İlk",
                            "last":       "Son",
                            "next":       "İleri",
                            "previous":   "Geri"
                        },
                    },
                    columnDefs: [
                        {"className": "dt-center", "targets": "_all"}
                    ],
                    buttons: [
                        'csv',
                        {
                            extend: 'excelHtml5',
                            autoFilter: true
                        },
                        {
                            extend:'pdfHtml5',
                            download:'open',
                            orientation:'landscape',
                            pageSize:'A4',
                            customize: function(doc) {
                                doc.content[1].margin = [ 100, 0, 100, 0 ], // left, top, right, bottom
                                doc.defaultStyle.fontSize = 11
                            }
                        }
                    ]
                });
                $(":button").removeClass("dt-button");
                $(":button").addClass("btn btn-secondary")
            });
        </script>""".replace("pageSize:'A4'", f"pageSize:'{pdf_sayfa}'").replace("paging: false", f"paging: {str(sayfalama).lower()}")

    return f"""<!doctype html>
<html lang="tr">
<head>
    <!-- Gerekli meta tagler -->
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0, shrink-to-fit=no">

    <!-- Bootstrap CSS - Font Awesome -->
    <link rel="stylesheet" href="https://cdn.jsdelivr.net/npm/@forevolve/bootstrap-dark@2.1.0/dist/css/bootstrap-dark.min.css" />
    <link rel="stylesheet" href="https://cdn.jsdelivr.net/npm/font-awesome@4.7.0/css/font-awesome.min.css">

    <!-- DataTables -->
    <link href="https://cdn.datatables.net/1.11.5/css/dataTables.bootstrap4.min.css" rel="stylesheet"/>
    <link href="https://cdn.datatables.net/buttons/2.2.2/css/buttons.dataTables.min.css" rel="stylesheet"/>

    <!-- CSS -->
    {css}

    <!-- Ajax -->
    <script src="https://ajax.googleapis.com/ajax/libs/jquery/3.6.0/jquery.min.js"></script>

    <title>{title}</title>
</head>
<body class="d-flex align-items-center min-vh-100">
    <div class="container">
        <div class="card">
            <div class="card-header">
                <i class="fa fa-info" aria-hidden="true"></i> | {header or title}
            </div>
            <div class="card-body">
                <h3 class="d-flex justify-content-around">{aciklama or title}</h3>
                <hr>
                <div class="table-responsive-sm">

{tablo_html}

                </div>
            </div>
            <div class="card-footer text-muted">
                <i class="fa fa-bed" aria-hidden="true"></i> | {footer or title}
            </div>
        </div>
    </div>

        <!-- jQuery ve Bootstrap Bundle (Popper içerir) -->
        <script src="https://cdn.jsdelivr.net/npm/jquery@3.6.0/dist/jquery.min.js"></script>
        <!-- <script src="https://cdn.jsdelivr.net/npm/popper.js@1.16.1/dist/umd/popper.min.js"></script> -->
        <script src="https://cdn.jsdelivr.net/npm/@forevolve/bootstrap-dark@2.1.0/dist/js/bootstrap.bundle.min.js"></script>

        <!-- DataTables -->
        <script src="https://cdn.datatables.net/1.11.5/js/jquery.dataTables.min.js"></script>
        <script src="https://cdn.datatables.net/1.11.5/js/dataTables.bootstrap4.min.js"></script>
        <script src="https://cdn.datatables.net/buttons/2.2.2/js/dataTables.buttons.min.js"></script>
        <script src="https://cdnjs.cloudflare.com/ajax/libs/jszip/3.1.3/jszip.min.js"></script>
        <script src="https://cdnjs.cloudflare.com/ajax/libs/pdfmake/0.1.53/pdfmake.min.js"></script>
        <script src="https://cdnjs.cloudflare.com/ajax/libs/pdfmake/0.1.53/vfs_fonts.js"></script>
        <script src="https://cdn.datatables.net/buttons/2.2.2/js/buttons.html5.min.js"></script>
        <script src="https://cdn.datatables.net/buttons/2.2.2/js/buttons.print.min.js"></script>

        <!-- Tablo Düzenle -->
        {javascript}
</body>
</html>"""


# Bu araç @keyiflerolsun tarafından | @KekikAkademi için yazılmıştır.

# * pip3 install -U PyAudio SpeechRecognition beepy gTTS playsound

from Kekik.cli          import konsol
from speech_recognition import Recognizer, Microphone, UnknownValueError
from beepy              import beep
from gtts               import gTTS
from playsound          import playsound
from os                 import system, remove

def ses2yazi(n_saniye_dinle:int | None, bip:bool=True) -> str:
    dinleyici = Recognizer()

    import os, sys, contextlib

    @contextlib.contextmanager
    def ignore_stderr():
        devnull = os.open(os.devnull, os.O_WRONLY)
        old_stderr = os.dup(2)
        sys.stderr.flush()
        os.dup2(devnull, 2)
        os.close(devnull)
        try:
            yield
        finally:
            os.dup2(old_stderr, 2)
            os.close(old_stderr)

    with ignore_stderr() as _, Microphone() as source:
        dinleyici.adjust_for_ambient_noise(source)

        if bip:
            beep()
        konsol.log("[purple][~] Mikrofon Dinleniyor..")

        try:
            if n_saniye_dinle:
                veri = dinleyici.record(source, duration=n_saniye_dinle)
            else:
                veri = dinleyici.listen(source, timeout=5)

            yazi = dinleyici.recognize_google(veri, language="tr")

            konsol.log("[magenta][~] Ses Erişimi Tamamlandı..")
        except UnknownValueError:
            yazi = ""
        except Exception as hata:
            konsol.log(f"[red][!] Hata: {type(hata).__name__}")
            yazi = ""

    return yazi

def yazi2ses(metin:str, mp3_adi:str) -> str:
    tts = gTTS(metin.strip(), lang="tr", slow=False)
    tts.save(f"{mp3_adi}.mp3")
    return f"{mp3_adi}.mp3"

def cevir(girdi_dosya:str, cikti_dosya:str) -> str:
    system(f'ffmpeg -hide_banner -loglevel error -y -i {girdi_dosya} -af "asetrate=44100*0.9, aresample=44100, atempo=1/1.15" {cikti_dosya}')
    remove(girdi_dosya)
    return cikti_dosya

def inceses(metin:str, cikti_adi:str) -> str:
    _gecici = yazi2ses(metin, "gecici")
    return cevir(_gecici, f"{cikti_adi}.mp3")


# ne_dedim = ses2yazi()
# print(ne_dedim)
# playsound(yazi2ses(ne_dedim, "dinlenen_ses"))
# playsound(inceses(ne_dedim, "dinlenen_ses"))


# Bu araç @keyiflerolsun tarafından | @KekikAkademi için yazılmıştır.

from pathlib import Path

def satir_ekle(dosya_adi:str, eklenecek_metin:str):
    """İstenen Dosyanın Sonuna Verilen Metni Satır Olarak Ekler"""

    # Dizin / Altdizin Yoksa Oluştur
    dizin = "/".join(dosya_adi.split("/")[:-1]) + "/"
    Path(dizin).mkdir(parents=True, exist_ok=True)

    # Dosyayı ekleme ve okuma modunda açın ('a+')
    with open(dosya_adi, "a+", encoding="utf-8") as dosya:

        dosya.seek(0)                       # Okuma imlecini dosyanın başına taşıyın

        veri = dosya.read(100)              # Dosya boş değilse '\n' ekleyin
        if len(veri) > 0:
            dosya.write("\n")

        dosya.write(eklenecek_metin)        # Dosyanın sonuna metin ekle

def satirlar_ekle(dosya_adi:str, eklenecek_metinler:list):
    """İstenen Dosyanın Sonuna Verilen Listeyi Satır Satır Ekler"""

    # Dizin / Altdizin Yoksa Oluştur
    dizin = "/".join(dosya_adi.split("/")[:-1]) + "/"
    Path(dizin).mkdir(parents=True, exist_ok=True)

    # Dosyayı ekleme ve okuma modunda açın ('a+')
    with open(dosya_adi, "a+", encoding="utf-8") as dosya:

        dosya.seek(0)                       # Okuma imlecini dosyanın başına taşıyın

        veri = dosya.read(100)              # Dosyanın boş olup olmadığını kontrol edin
        satir_atla = len(veri) > 0
        for satir in eklenecek_metinler:    # Listedeki her elemanı gez
            if satir_atla:                  # Dosya boş değilse, ilk satırın başına '\n' ekleyin
                dosya.write("\n")
            else:                           # diğer satırlar her zaman satır eklemeden önce '\n' ekler
                satir_atla = True

            dosya.write(satir)              # Dosyanın sonuna eleman ekle

def satir_sil(dosya:str, metin:str) -> None:
    """İlgili Metnin Geçtiği Satırı İstenen Dosyanın İçerisinden Siler"""

    # Dosyayı yeniden yazılabilir modunda açın ('r+')
    with open(dosya, "r+", encoding="utf-8") as dosya:
        satirlar = dosya.readlines()            # Satırları liste olarak alın
        dosya.seek(0)                           # Okuma imlecini dosyanın başına taşıyın
        for satir in satirlar:                  # Satırları gezin
            if metin not in satir and \
                satir !='\n':                   # Eğer metin satır içinde geçmiyorsa ve \n Değilse
                dosya.write(satir)              # Dosyaya ekleyin
        dosya.truncate()                        # Var olan dosyadakiş tüm verileri silin


# Bu araç @keyiflerolsun tarafından | @KekikAkademi için yazılmıştır.

def n_adete_bol(liste:list, adet:int=2) -> list[list]:
    return [liste[say::adet] for say in range(adet)]

def n_er_hale_getir(liste:list, adet:int=2) -> list[list]:
    return [liste[i : i + adet] for i in range(0, len(liste), adet)]


# Bu araç @keyiflerolsun tarafından | @KekikAkademi için yazılmıştır.

from typing                      import Literal
from Kekik.kisi_ver.isimler      import en_isimler, tr_isimler
from Kekik.kisi_ver.soyisimler   import en_soyisimler, tr_soyisimler
from Kekik.kisi_ver.biyografiler import en_biyografiler, tr_biyografiler
from random                      import choice, randint
# from thispersondoesnotexist      import save_online_person
from Kekik                       import slugify
from pathlib                     import Path

async def kisi_ver(dil:Literal["tr", "en"], dizin:str="tmp") -> dict[str, str]:
    match dil:
        case "en":
            isim      = choice(en_isimler)
            soyisim   = choice(en_soyisimler)
            biyografi = choice(en_biyografiler)
        case "tr":
            isim      = choice(tr_isimler)
            soyisim   = choice(tr_soyisimler)
            biyografi = choice(tr_biyografiler)

    kullanici_adi = f"{slugify(isim[:4]).title()}{randint(0,99)}{slugify(soyisim[:4]).title()}"

    # Path(dizin).mkdir(parents=True, exist_ok=True)
    # await save_online_person(f"{dizin}/{kullanici_adi}.jpg")

    return {
        "isim"          : isim,
        "soyisim"       : soyisim,
        "kullanici_adi" : kullanici_adi,
        "biyografi"     : biyografi,
        # "profil_resmi"  : f"{dizin}/{kullanici_adi}.jpg"
    }


# https://github.com/Skuzzy_xD/TelePyroBot

def okunabilir_byte(boyut: int) -> str:
    """baytları okunabilir biçime dönüştürür.

    Arg:
        boyut (int):
            12345645123456

    Return:
        (str):
            "135 MB"
    """
    if not boyut:
        return ""

    # https://stackoverflow.com/a/49361727/4723940
    # 2**10 = 1024
    binyirmidort = 2 ** 10

    say = 0
    cikti_sozluk = {0: " ", 1: "K", 2: "M", 3: "G", 4: "T"}

    while boyut > binyirmidort:
        boyut /= binyirmidort
        say += 1

    return f'{str(round(boyut, 2))} {cikti_sozluk[say]}B'


# Bu araç @keyiflerolsun tarafından | @KekikAkademi için yazılmıştır.

#---------------------------------------------------#
from signal import signal, SIGINT

def sinyal_yakala(signal, frame):
    cikis_yap()

signal(SIGINT, sinyal_yakala)

# from warnings import filterwarnings, simplefilter
# filterwarnings("ignore")
# simplefilter("ignore")

# import sys, logging
# logging.disable(sys.maxsize)

# import asyncio, platform
# if platform.system() == "Windows":
#     asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())
#---------------------------------------------------#

from rich import pretty, traceback
pretty.install()
traceback.install(show_locals=False)

from rich.console import Console

konsol = Console(log_path=False)

#---------------------------------------------------#
import os, platform

if os.name == "nt":
    kullanici_adi = os.getlogin()
else:
    import pwd
    kullanici_adi = pwd.getpwuid(os.geteuid())[0]

oturum = f"{kullanici_adi}@{platform.node()}"

def temizle():
    if platform.system() == "Windows":
        os.system("cls")
    else:
        os.system("clear")

konum   = os.getcwd().split("\\") if platform.system() == "Windows" else os.getcwd().split("/")
secenek = lambda : konsol.input(f"[red]{oturum}:[/][cyan]~/../{konum[-2]}/{konum[-1]} >> ")

def hata_salla(hata:Exception) -> None:
    "Yakalanan Exception'ı ekranda gösterir.."

    konsol.print(f"[bold yellow2]{type(hata).__name__}[/] [bold magenta]||[/] [bold grey74]{hata}[/]", width=70, justify="center")

#---------------------------------------------------#
from contextlib import suppress
from pathlib    import Path
from sys        import exit
from shutil     import rmtree
from traceback  import format_exc

def bellek_temizle():
    with suppress(Exception):
        [alt_dizin.unlink() for alt_dizin in Path(".").rglob("*.py[coi]")]
        [alt_dizin.rmdir()  for alt_dizin in Path(".").rglob("__pycache__")]
        [rmtree(alt_dizin)  for alt_dizin in Path(".").rglob("*.build")]
        [alt_dizin.unlink() for alt_dizin in Path(".").rglob("*.bak")]

bellek_temizle()

def cikis_yap(_print=True):
    if _print:
        konsol.print("\n\n")
        konsol.log("[bold purple]Çıkış Yapıldı..")
    bellek_temizle()
    exit()

def hata_yakala(hata:Exception):
    if (hata in {KeyboardInterrupt, SystemExit, EOFError, RuntimeError}) or (str(hata).startswith(("'coroutine' object is not iterable", "'KekikT"))):
        cikis_yap()
    konsol.print(f"\n\n[bold red]{format_exc()}")
    cikis_yap()

def log_salla(sol:str, orta:str, sag:str) -> None:
    "Sol orta ve sağ şeklinde ekranda hizalanmış tek satır log verir.."

    sol  = f"{sol[:13]}[bright_blue]~[/]"  if len(sol)  > 14 else sol
    orta = f"{orta[:19]}[bright_blue]~[/]" if len(orta) > 20 else orta
    sag  = f"{sag[:14]}[bright_blue]~[/]"  if len(sag)  > 15 else sag

    bicimlendir = f"[bold red]{sol:14}[/] [green]||[/] [yellow]{orta:20}[/] {'':>2}[green]||[/] [magenta]{sag:^16}[/]"
    konsol.log(bicimlendir)